import csv
from colorama import Fore, Back, init
from config import login_credentials_filename, cr
from cryptor import decrypt_user_credentials

init()

def output_login_credentials():
    """Show saved credentials"""
    try:
        with open(login_credentials_filename) as f:
            csv_reader = csv.DictReader(f)
            field_names = '\t{:<20} {:<20} {:<20} {:<20}'.format(
                'Title','Website','Username','Password')
            i = 0
            print(Fore.BLACK+Back.WHITE+field_names+cr)
            for line in csv_reader:
                i = i + 1
                title = line['Title']
                website = line['Website']
                username = line['Username']
                password = line['Password']
                password = decrypt_user_credentials(password)
                final_ = '{:<20} {:<20} {:<20} {:<20}'.format(title,website,
                                                            username,password)

                print(f'\n{Fore.WHITE+Back.BLACK+str(i)}.\t{final_+cr}')
    except FileNotFoundError:
        print(Fore.LIGHTRED_EX+'\nNo login credentials were found!\n'+cr)
        pass


"""Python password manger main menu"""
# Dependencies
from colorama import Fore, Back, init
# Local modules
from keys import generate_key, enter_key
from input import input_login_credentials
from output import output_login_credentials
from password import generate_password
from reset import delete_all
from config import login_credentials_filename, cr, key_filename

init()

if __name__ == '__main__':
    print(f"\n{Fore.BLACK+Back.LIGHTYELLOW_EX+'Pysafe 0.2.0 inbound!'+cr}"
          f"\nCheck out the {Fore.LIGHTRED_EX+'README.md'+cr} "
          f"file for info on how to use this program.")
    while True:
        print('\n'+Fore.BLACK+Back.LIGHTCYAN_EX+'What do you wish to do?'+cr)
        user_choice = input(
                            "1.Generate a new encryption key"
                            "\n2.Use previous encryption key"
                            "\n3.Enter a new login"
                            "\n4.Show saved login credentials"
                            "\n5.Generate password"
                            "\n6.Show locations of saved files"
                            "\n7.Delete all data"
                            "\n0.Exit\n")

        if user_choice == '0':
            exit()
        elif user_choice == '1':
            generate_key()
            continue
        elif user_choice == '2':
            enter_key()
            continue
        elif user_choice == '3':
            input_login_credentials()
            continue
        elif user_choice == '4':
            output_login_credentials()
            continue
        elif user_choice == '5':
            generate_password()
            continue
        elif user_choice == '6':
            print(f'Encryption key file path:'
                  f'{Fore.LIGHTYELLOW_EX}{key_filename}'+cr)
            print(f'Login credentials file path:'
                  f'{Fore.LIGHTYELLOW_EX}{login_credentials_filename}'+cr)
        elif user_choice == '7':
            delete_all()
        else:
            print('\n'+Fore.LIGHTRED_EX+'Please enter a valid option!'+cr)
            continue





import json
from colorama import Fore, init
from cryptography.fernet import Fernet
from config import key_filename, cr

init()

def generate_key():
    """Generate a new encryption key"""
    while True:
        print(f'Are you sure you want to generate a new encryption key? '
        f'{Fore.LIGHTRED_EX}This will erase any existed encryption key?{cr}'
        f'(y/n)')
        user_choice = input()
        if user_choice.lower() == 'y':
            key = Fernet.generate_key()
            key = key.decode()
            print(f'\nEncryption key:{Fore.LIGHTYELLOW_EX+(key)}\n'+cr)
            save_key_to_file(key)
            return key
        elif user_choice.lower() == 'n':
            break
        else:
            print('Please enter a enter a valid option!')
            continue

def save_key_to_file(key):
    """Save generated key to file"""
    with open(key_filename,'w') as f:
        json.dump(key, f,)
        #f.write(key)
        print(f"\nThe encryption key has been saved to" +
              f"{Fore.LIGHTYELLOW_EX+key_filename}"+cr)

def get_key():
    """Read key from file"""
    with open(key_filename, 'r') as key_object:
        key = json.load(key_object)
        key = key.encode()
        return key

def enter_key():
    """Input key from user"""
    key = input('Enter encryption key:')
    if len(key) == 46 and key[44] == '=':
        save_key_to_file(key)
        key = key.encode()
        return key
    else:
        print(Fore.LIGHTRED_EX+'\nYou entered an invalid key!')




# Dependencies
from colorama import Fore, init
from cryptography.fernet import Fernet
# Local modules
from keys import get_key, generate_key, enter_key
from config import cr

init()

def encrypt_credentials(password):
    """Encrypt credentials using the provided key"""
    try:
        key = get_key()
        f_1 = Fernet(key)
        token = f_1.encrypt(password.encode())
        token = token.decode()
        return token

    except FileNotFoundError:
        print(Fore.LIGHTRED_EX+'No encryption key was found!\n'+cr)

        while True:
            user_choice = input('(Generate key)(Input key)(cancel)?(g)(i)(c)')
            if user_choice.lower() == 'c':
                print(Fore.LIGHTRED_EX+'You need an encryption key to '
                    'encrypt your login credentials!\nBack to the main '
                                       'menu!\n'+cr)
                break

            elif user_choice.lower() == 'g':
                key = generate_key()
                f_1 = Fernet(key)
                token = f_1.encrypt(password.encode())
                token = token.decode()
                return token

            elif user_choice.lower()=='i':
                key = enter_key()
                f_1 = Fernet(key)
                token = f_1.encrypt(password.encode())
                token = token.decode()
                return token
            else:
                print(Fore.LIGHTRED_EX+'Please enter a valid option!'+cr)
                continue




def decrypt_user_credentials(token):
    """Decrypt saved credentials"""
    try:
        key = get_key()
        f_1 = Fernet(key)
        token = bytes(token,'utf-8')
        password = f_1.decrypt(token)
        password = password.decode()
        return password

    except FileNotFoundError:
        print(Fore.LIGHTRED_EX+'No encryption key was found')
        while True:
            user_choice = input('Enter encryption key?(y/n)')
            if user_choice.lower() == 'y':
                key = enter_key()
                f_1 = Fernet(key)
                token = bytes(token,'utf-8')
                password = f_1.decrypt(token)
                password = password.decode()
                return password



import csv
from colorama import Fore, Back, init
from config import login_credentials_filename, field_names, cr
from password import input_password

init()

def input_login_credentials():
    """Accept user input"""
    title = input('Website title:')
    website = input('Website link:')
    username = input('Username/email:')
    password = input_password()
    save_user_input(title, website, username, password)

def save_user_input(title,website,username,password):
    """Save credentials to a CSV file"""
    try:
        if open(login_credentials_filename):#Append to file without header
            with open(login_credentials_filename,'a+',newline='') as f:
                thewriter = csv.DictWriter(f,fieldnames=field_names)
                thewriter.writerow({'Title':title,'Website':website,'Username':
                                                username,'Password':password})
                print(f"{Fore.BLACK+Back.GREEN}Saving was successful"+cr)

    except FileNotFoundError:   #Create new file with header
        with open(login_credentials_filename,'w+',newline='') as f:
            thewriter = csv.DictWriter(f,fieldnames=field_names)
            thewriter.writeheader()
            thewriter.writerow({'Title':title,'Website': website,'Username':
                                             username,'Password': password})
            print(f"{Fore.BLACK+Back.GREEN}Saving was successful!"+cr)




# Builtin modules
import random
import string
from sys import platform
from getpass import getpass
# Dependencies
from colorama import Fore, init
# Local modules
from cryptor import encrypt_credentials
from config import cr

init()

def generate_password():
    """Generate password with a given length"""
    while True:
        length = input('Password length:')
        try:
            length = int(length)
        except ValueError:
            print(Fore.LIGHTRED_EX+"You didn't enter a number"+cr)
            continue
        else:
            custom_punctuation = "!#$%*+-?@_"
            password_mix = string.ascii_letters+string.digits+custom_punctuation
            p = []
            i = 0
            while i <= length:
                p.append(random.choice(password_mix))
                i = i + 1
            separator =''
            print(f'\nNew password:'
                  f'{Fore.LIGHTYELLOW_EX+(separator.join(p))}\n'+cr)
            while True:
                print('Regenerate password?(y/n)')
                regenerate = input()
                if regenerate.lower() == 'y':
                    break
                elif regenerate.lower() == 'n':
                    return (separator.join(p))
                else:
                    print(Fore.LIGHTRED_EX + "Enter a valid option!"+cr)
                    continue

def input_password():
    """Accept password from user"""
    while True:
        gen_password = input('Generate password / Save password?(g/s):')
        if gen_password.lower() == 'g':
            plain_password = generate_password()
            password = encrypt_credentials(plain_password)
            return password
        elif gen_password.lower() == 's':
            if platform == 'win32':
                print(Fore.LIGHTRED_EX+'Mouse right-click to paste '
                                       'passwords!'+cr)
            else:
                plain_password = getpass('New password:')
                password = encrypt_credentials(plain_password)
                return password
        else:
            print(Fore.LIGHTRED_EX+'Enter a valid option!'+cr)
            continue




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pysafe",
    version="0.2.0",
    author="MNKANOUT",
    author_email="mnkanout@gmail.com",
    description="Python password manger",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MNKanout/Pysafe",
    package_dir = {'':'src'},
    py_modules = ['pysafe','cryptor','password','input',
                  'output','keys','reset'],
    scripts=["src/pysafe.py"],
    install_requires=[
        'cffi==1.14.0',
        'colorama==0.4.3',
        'cryptography==2.8',
        'pycparser==2.20',
        'six==1.14.0',

      ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Natural Language :: English",
    ],
    python_requires='>=3.7',
)


# Builtin modules
import os
# Dependencies
from colorama import Fore, Back, init
# Local modules
from config import login_credentials_filename, key_filename, cr

init()

def delete_all():
    """Delete credentials and keys"""
    confirm = input("Delete saved encryption key and login credentials?(y/n)\t")
    if confirm.lower() == 'y':
        try:
            os.remove(key_filename)
        except FileNotFoundError:
            print(Fore.LIGHTRED_EX +'No encryption key was found\n'+cr)
        else:
            print(Fore.BLACK+Back.GREEN +'Encryption key has been '
                                         'deleted!'+cr+'\n')
        try:
            os.remove(login_credentials_filename)
        except FileNotFoundError:
            print(Fore.LIGHTRED_EX +'No login credentials were found'+cr+'\n')
        else:
            print(Fore.BLACK + Back.GREEN +'All login Credentials have been'
                                           ' deleted!'+cr+'\n')


    elif confirm.lower() =='n':
        print(Fore.BLACK+Back.GREEN+'No key nor login credentials got '
                                    'deleted'+cr+'\n')
        pass

    else:
        print(Fore.LIGHTRED_EX +'No valid option got selected, back to the '
                                'main menu!'+cr+'\n')
        pass